@@ -1,15 +1,16 @@
 import sys
 
 import kubernetes
-from dagster import DagsterInvariantViolationError, EventMetadataEntry, Field, Noneable, check
+from dagster import DagsterInvariantViolationError, MetadataEntry, check
 from dagster.config.field import resolve_to_config_type
 from dagster.config.validate import process_config
 from dagster.core.events import EngineEventData
 from dagster.core.execution.retries import RetryMode
 from dagster.core.launcher import LaunchRunContext, RunLauncher
-from dagster.core.storage.pipeline_run import PipelineRunStatus
+from dagster.core.launcher.base import CheckRunHealthResult, WorkerStatus
+from dagster.core.storage.pipeline_run import PipelineRun, PipelineRunStatus
 from dagster.core.storage.tags import DOCKER_IMAGE_TAG
-from dagster.serdes import ConfigurableClass, ConfigurableClassData, serialize_dagster_namedtuple
+from dagster.serdes import ConfigurableClass, ConfigurableClassData
 from dagster.utils import frozentags, merge_dicts
 from dagster.utils.error import serializable_error_info_from_exc_info
 from dagster_k8s.job import (
@@ -20,7 +21,7 @@ from dagster_k8s.job import (
 )
 from dagster_k8s.utils import delete_job
 
-from .config import CELERY_K8S_CONFIG_KEY, celery_k8s_config
+from .config import CELERY_K8S_CONFIG_KEY, celery_k8s_executor_config
 
 
 class CeleryK8sRunLauncher(RunLauncher, ConfigurableClass):
@@ -36,7 +37,7 @@ class CeleryK8sRunLauncher(RunLauncher, ConfigurableClass):
            and each step execution spawns a step execution Kubernetes Job. See the implementation
            defined in :py:func:`dagster_celery_k8.executor.create_k8s_job_task`.
 
-    You may configure a Dagster instance to use this RunLauncher by adding a section to your
+    You can configure a Dagster instance to use this RunLauncher by adding a section to your
     ``dagster.yaml`` like the following:
 
     .. code-block:: yaml
@@ -51,32 +52,6 @@ class CeleryK8sRunLauncher(RunLauncher, ConfigurableClass):
             broker: "some_celery_broker_url"
             backend: "some_celery_backend_url"
 
-    As always when using a :py:class:`~dagster.serdes.ConfigurableClass`, the values
-    under the ``config`` key of this YAML block will be passed to the constructor. The full list
-    of acceptable values is given below by the constructor args.
-
-    Args:
-        instance_config_map (str): The ``name`` of an existing Volume to mount into the pod in
-            order to provide a ConfigMap for the Dagster instance. This Volume should contain a
-            ``dagster.yaml`` with appropriate values for run storage, event log storage, etc.
-        dagster_home (str): The location of DAGSTER_HOME in the Job container; this is where the
-            ``dagster.yaml`` file will be mounted from the instance ConfigMap specified above.
-        postgres_password_secret (str): The name of the Kubernetes Secret where the postgres
-            password can be retrieved. Will be mounted and supplied as an environment variable to
-            the Job Pod.
-        load_incluster_config (Optional[bool]):  Set this value if you are running the launcher
-            within a k8s cluster. If ``True``, we assume the launcher is running within the target
-            cluster and load config using ``kubernetes.config.load_incluster_config``. Otherwise,
-            we will use the k8s config specified in ``kubeconfig_file`` (using
-            ``kubernetes.config.load_kube_config``) or fall back to the default kubeconfig. Default:
-            ``True``.
-        kubeconfig_file (Optional[str]): The kubeconfig file from which to load config. Defaults to
-            None (using the default kubeconfig).
-        broker (Optional[str]): The URL of the Celery broker.
-        backend (Optional[str]): The URL of the Celery backend.
-        include (List[str]): List of includes for the Celery workers
-        config_source: (Optional[dict]): Additional settings for the Celery app.
-        retries: (Optional[dict]): Default retry configuration for Celery tasks.
     """
 
     def __init__(
@@ -93,6 +68,15 @@ class CeleryK8sRunLauncher(RunLauncher, ConfigurableClass):
         retries=None,
         inst_data=None,
         k8s_client_batch_api=None,
+        env_config_maps=None,
+        env_secrets=None,
+        volume_mounts=None,
+        volumes=None,
+        service_account_name=None,
+        image_pull_policy=None,
+        image_pull_secrets=None,
+        labels=None,
+        fail_pod_on_run_failure=None,
     ):
         self._inst_data = check.opt_inst_param(inst_data, "inst_data", ConfigurableClassData)
 
@@ -120,6 +104,29 @@ class CeleryK8sRunLauncher(RunLauncher, ConfigurableClass):
 
         retries = check.opt_dict_param(retries, "retries") or {"enabled": {}}
         self.retries = RetryMode.from_config(retries)
+
+        self._env_config_maps = check.opt_list_param(
+            env_config_maps, "env_config_maps", of_type=str
+        )
+        self._env_secrets = check.opt_list_param(env_secrets, "env_secrets", of_type=str)
+
+        self._volume_mounts = check.opt_list_param(volume_mounts, "volume_mounts")
+        self._volumes = check.opt_list_param(volumes, "volumes")
+
+        self._service_account_name = check.opt_str_param(
+            service_account_name, "service_account_name"
+        )
+        self._image_pull_policy = check.opt_str_param(
+            image_pull_policy, "image_pull_policy", "IfNotPresent"
+        )
+        self._image_pull_secrets = check.opt_list_param(
+            image_pull_secrets, "image_pull_secrets", of_type=dict
+        )
+        self._labels = check.opt_dict_param(labels, "labels", key_type=str, value_type=str)
+        self._fail_pod_on_run_failure = check.opt_bool_param(
+            fail_pod_on_run_failure, "fail_pod_on_run_failure"
+        )
+
         super().__init__()
 
     @property
@@ -128,20 +135,9 @@ class CeleryK8sRunLauncher(RunLauncher, ConfigurableClass):
 
     @classmethod
     def config_type(cls):
-        """Include all arguments required for DagsterK8sJobConfig along with additional arguments
-        needed for the RunLauncher itself.
-        """
         from dagster_celery.executor import CELERY_CONFIG
 
-        job_cfg = DagsterK8sJobConfig.config_type_run_launcher()
-
-        run_launcher_extra_cfg = {
-            "load_incluster_config": Field(bool, is_required=False, default_value=True),
-            "kubeconfig_file": Field(Noneable(str), is_required=False, default_value=None),
-        }
-
-        res = merge_dicts(job_cfg, run_launcher_extra_cfg)
-        return merge_dicts(res, CELERY_CONFIG)
+        return merge_dicts(DagsterK8sJobConfig.config_type_run_launcher(), CELERY_CONFIG)
 
     @classmethod
     def from_config_value(cls, inst_data, config_value):
@@ -190,19 +186,7 @@ class CeleryK8sRunLauncher(RunLauncher, ConfigurableClass):
 
             job_image = job_image_from_executor_config
 
-        job_config = DagsterK8sJobConfig(
-            dagster_home=self.dagster_home,
-            instance_config_map=self.instance_config_map,
-            postgres_password_secret=self.postgres_password_secret,
-            job_image=check.str_param(job_image, "job_image"),
-            image_pull_policy=exc_config.get("image_pull_policy"),
-            image_pull_secrets=exc_config.get("image_pull_secrets"),
-            service_account_name=exc_config.get("service_account_name"),
-            env_config_maps=exc_config.get("env_config_maps"),
-            env_secrets=exc_config.get("env_secrets"),
-            volume_mounts=exc_config.get("volume_mounts"),
-            volumes=exc_config.get("volumes"),
-        )
+        job_config = self.get_k8s_job_config(job_image, exc_config)
 
         self._instance.add_run_tags(
             run.run_id,
@@ -213,24 +197,24 @@ class CeleryK8sRunLauncher(RunLauncher, ConfigurableClass):
 
         from dagster.cli.api import ExecuteRunArgs
 
-        input_json = serialize_dagster_namedtuple(
-            # depends on DagsterInstance.get() returning the same instance
-            # https://github.com/dagster-io/dagster/issues/2757
-            ExecuteRunArgs(
-                pipeline_origin=pipeline_origin,
-                pipeline_run_id=run.run_id,
-                instance_ref=None,
-            )
-        )
+        run_args = ExecuteRunArgs(
+            pipeline_origin=pipeline_origin,
+            pipeline_run_id=run.run_id,
+            instance_ref=self._instance.get_ref(),
+            set_exit_code_on_failure=self._fail_pod_on_run_failure,
+        ).get_command_args()
 
         job = construct_dagster_k8s_job(
             job_config,
-            args=["dagster", "api", "execute_run", input_json],
+            args=run_args,
             job_name=job_name,
             pod_name=pod_name,
-            component="run_coordinator",
+            component="run_worker",
             user_defined_k8s_config=user_defined_k8s_config,
             env_vars=env_vars,
+            labels={
+                "dagster/job": pipeline_origin.pipeline_name,
+            },
         )
 
         job_namespace = exc_config.get("job_namespace")
@@ -240,9 +224,9 @@ class CeleryK8sRunLauncher(RunLauncher, ConfigurableClass):
             run,
             EngineEventData(
                 [
-                    EventMetadataEntry.text(job_name, "Kubernetes Job name"),
-                    EventMetadataEntry.text(job_namespace, "Kubernetes Namespace"),
-                    EventMetadataEntry.text(run.run_id, "Run ID"),
+                    MetadataEntry.text(job_name, "Kubernetes Job name"),
+                    MetadataEntry.text(job_namespace, "Kubernetes Namespace"),
+                    MetadataEntry.text(run.run_id, "Run ID"),
                 ]
             ),
             cls=self.__class__,
@@ -254,14 +238,30 @@ class CeleryK8sRunLauncher(RunLauncher, ConfigurableClass):
             run,
             EngineEventData(
                 [
-                    EventMetadataEntry.text(job_name, "Kubernetes Job name"),
-                    EventMetadataEntry.text(job_namespace, "Kubernetes Namespace"),
-                    EventMetadataEntry.text(run.run_id, "Run ID"),
+                    MetadataEntry.text(job_name, "Kubernetes Job name"),
+                    MetadataEntry.text(job_namespace, "Kubernetes Namespace"),
+                    MetadataEntry.text(run.run_id, "Run ID"),
                 ]
             ),
             cls=self.__class__,
         )
 
+    def get_k8s_job_config(self, job_image, exc_config):
+        return DagsterK8sJobConfig(
+            dagster_home=self.dagster_home,
+            instance_config_map=self.instance_config_map,
+            postgres_password_secret=self.postgres_password_secret,
+            job_image=check.opt_str_param(job_image, "job_image"),
+            image_pull_policy=exc_config.get("image_pull_policy", self._image_pull_policy),
+            image_pull_secrets=exc_config.get("image_pull_secrets", []) + self._image_pull_secrets,
+            service_account_name=exc_config.get("service_account_name", self._service_account_name),
+            env_config_maps=exc_config.get("env_config_maps", []) + self._env_config_maps,
+            env_secrets=exc_config.get("env_secrets", []) + self._env_secrets,
+            volume_mounts=exc_config.get("volume_mounts", []) + self._volume_mounts,
+            volumes=exc_config.get("volumes", []) + self._volumes,
+            labels=merge_dicts(self._labels, exc_config.get("labels", {})),
+        )
+
     # https://github.com/dagster-io/dagster/issues/2741
     def can_terminate(self, run_id):
         check.str_param(run_id, "run_id")
@@ -316,7 +316,7 @@ class CeleryK8sRunLauncher(RunLauncher, ConfigurableClass):
                     cls=self.__class__,
                 )
             return termination_result
-        except Exception:  # pylint: disable=broad-except
+        except Exception:
             self._instance.report_engine_event(
                 message="Dagster Job was not terminated successfully; encountered error in delete_job",
                 pipeline_run=run,
@@ -334,27 +334,51 @@ class CeleryK8sRunLauncher(RunLauncher, ConfigurableClass):
         executor_config = _get_validated_celery_k8s_executor_config(run_config)
         return executor_config.get("job_namespace")
 
+    @property
+    def supports_check_run_worker_health(self):
+        return True
+
+    def check_run_worker_health(self, run: PipelineRun):
+        job_namespace = _get_validated_celery_k8s_executor_config(run.run_config).get(
+            "job_namespace"
+        )
+        job_name = get_job_name_from_run_id(run.run_id)
+        try:
+            job = self._batch_api.read_namespaced_job(namespace=job_namespace, name=job_name)
+        except Exception:
+            return CheckRunHealthResult(
+                WorkerStatus.UNKNOWN, str(serializable_error_info_from_exc_info(sys.exc_info()))
+            )
+        if job.status.failed:
+            return CheckRunHealthResult(WorkerStatus.FAILED, "K8s job failed")
+        return CheckRunHealthResult(WorkerStatus.RUNNING)
+
 
 def _get_validated_celery_k8s_executor_config(run_config):
     check.dict_param(run_config, "run_config")
 
     executor_config = run_config.get("execution", {})
+    execution_config_schema = resolve_to_config_type(celery_k8s_executor_config())
+
+    # In run config on jobs, we don't have an executor key
     if not CELERY_K8S_CONFIG_KEY in executor_config:
-        raise DagsterInvariantViolationError(
-            "{config_key} execution configuration must be present in the run config to use the CeleryK8sRunLauncher. "
-            "Note: You may also be seeing this error because you are using the configured API. "
-            "Using configured with the {config_key} executor is not supported at this time, "
-            "and all executor config must be directly in the run config without using configured.".format(
-                config_key=CELERY_K8S_CONFIG_KEY,
-            ),
+
+        execution_run_config = executor_config.get("config", {})
+    else:
+        execution_run_config = (run_config["execution"][CELERY_K8S_CONFIG_KEY] or {}).get(
+            "config", {}
         )
 
-    execution_config_schema = resolve_to_config_type(celery_k8s_config())
-    execution_run_config = (run_config["execution"][CELERY_K8S_CONFIG_KEY] or {}).get("config", {})
     res = process_config(execution_config_schema, execution_run_config)
 
     check.invariant(
-        res.success, "Incorrect {} execution schema provided".format(CELERY_K8S_CONFIG_KEY)
+        res.success,
+        "Incorrect execution schema provided. Note: You may also be seeing this error "
+        "because you are using the configured API. "
+        "Using configured with the {config_key} executor is not supported at this time, "
+        "and all executor config must be directly in the run config without using configured.".format(
+            config_key=CELERY_K8S_CONFIG_KEY,
+        ),
     )
 
     return res.value
