@@ -1,7 +1,6 @@
-from dagster_celery_k8s import CeleryK8sRunLauncher, celery_k8s_job_executor
-from dagster_celery_k8s_tests.example_celery_mode_def import celery_enabled_pipeline
-
 from dagster import ExecutorDefinition
+from dagster_celery_k8s import CeleryK8sRunLauncher, celery_k8s_job_executor
+from dagster_celery_k8s_tests.example_celery_mode_def import celery_enabled_job
 
 
 def test_include_launcher_works():
@@ -13,4 +12,4 @@ def test_include_executor_works():
 
 
 def test_example_celery_mode_def():
-    assert celery_enabled_pipeline
+    assert celery_enabled_job
